# Copyright 2020 Mufeed Ali
# Copyright 2020 Rafael Mardojai CM
# SPDX-License-Identifier: GPL-3.0-or-later

from dialect.widgets.lang_selector import LangSelector  # noqa
from dialect.widgets.provider_preferences import ProviderPreferences  # noqa
from dialect.widgets.speech_button import SpeechButton  # noqa
from dialect.widgets.textview import TextView  # noqa
from dialect.widgets.theme_switcher import ThemeSwitcher  # noqa
